"""
This project allows sending STMP email over TLS using SSL.
Use the Emailer class to create an instance for an outgoing email account,
and call the send method to send an email. You can also use the EmailHandler
class to add a custom email handler to your app's logger.
"""

__version__ = "1.0.1"

from .sendmail import *



from __future__ import annotations

import mimetypes
import platform
import smtplib
import ssl
import traceback
from email.message import EmailMessage
from logging import LogRecord
from logging.handlers import SMTPHandler
from pathlib import Path
from typing import Optional, Union

try:
    import keyring
except ImportError:
    pass

machine = platform.node().lower()


class Emailer:
    def __init__(
        self,
        secure_host: str,
        port: int,
        email_address: str,
        password: Optional[str] = None,
        user: Optional[str] = None,
        unsecure_host: str = "",
        use_keyring: bool = False,
        include_ssl_status: bool = False,
    ) -> None:
        self.secure_host: str = secure_host
        self.include_ssl_status: bool = include_ssl_status
        self.bu_host: str = unsecure_host
        self.port: int = port
        self.email_address: str = email_address
        self.user: str = user or email_address
        self.password: str = password or ""
        if use_keyring:
            self.password = keyring.get_password(self.email_address, self.user) or ""
            pass

    def __send(
        self,
        to: Union[str, list[str]],
        subject: str,
        body: str,
        attachments: list[str] = [],
        cc_list: list[str] = [],
        bcc_list: list[str] = [],
        html: bool = False,
        secure: bool = True,
    ) -> str:
        """
        Sends STMP email over TLS using SSL

        params
        ------
        to (str|list[str]): email recipient(s)
        subject (str): email subject line
        body (str): body of email as text or HTML string
        cc_list (list[str]): recipients to CC (default: [])
        bcc_list (list[str]): recipients to BCC (default: [])
        html (bool): whether to send body as HTML (default: False)
        secure (bool): whether to send with SSL (default: True)

        returns
        -------
        response (subprocess.CompletedProcess): exit code returned by cmd.exe after executing
            email send command
        """

        status_str: str = "SSL" if secure else "non-SSL backup host"
        newline: str = "\n"
        if self.include_ssl_status:
            if html:
                body += f"<p>Email sent over {status_str}</p>"
            else:
                body += f"{newline}{newline}*** Email sent over {status_str} ***"
        to = to if isinstance(to, str) else ",".join(to)

        smtp_server = self.secure_host if secure else self.bu_host
        msg = EmailMessage()
        if html:
            msg.set_content(body, subtype="html")
        else:
            msg.set_content(body)
        msg["Subject"] = subject
        msg["From"] = self.email_address
        msg["To"] = to
        if cc_list is not None:
            msg["CC"] = ",".join(cc_list)
        if bcc_list is not None:
            msg["BCC"] = ",".join(bcc_list)
        for attachment in attachments:
            filename = Path(attachment).name
            mimetype, _ = mimetypes.guess_type(attachment)
            if mimetype is None:
                continue
            maintype, subtype = mimetype.split("/")
            readmode = "r" if maintype == "text" else "rb"
            kwargs = {"filename": filename}
            if maintype != "text":
                kwargs["maintype"] = maintype
                kwargs["subtype"] = subtype
            with open(attachment, readmode) as f:
                content = f.read()
                msg.add_attachment(content, **kwargs)  # type: ignore
        if secure:
            context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE
        try:
            with smtplib.SMTP(smtp_server, self.port) as server:
                if secure:
                    server.starttls(context=context)
                    server.login(self.email_address, self.password)
                server.send_message(msg)
                return "success"
        except Exception as e:
            return f"{e}: {traceback.format_exc}"

    def send(
        self,
        to: Union[str, list[str]],
        subject: str,
        body: str,
        attachments: list[str] = [],
        cc_list: list[str] = [],
        bcc_list: list[str] = [],
        html: bool = False,
    ) -> str:
        """
        Sends STMP email over TLS using SSL

        params
        ------
        to (str|list[str]): email recipient(s)
        subject (str): email subject line
        body (str): body of email as text or HTML string
        cc_list (list[str]): recipients to CC (default: [])
        bcc_list (list[str]): recipients to BCC (default: [])
        html (bool): whether to send body as HTML (default: False)

        returns
        -------
        response (subprocess.CompletedProcess): exit code returned by cmd.exe after executing
            email send command
        """
        args = [to, subject, body]
        kwargs = {
            "attachments": attachments,
            "cc_list": cc_list,
            "bcc_list": bcc_list,
            "html": html,
        }
        response = self.__send(*args, **kwargs)
        if response != "success":
            response = self.__send(*args, **kwargs, secure=False)
        return response


class EmailHandler(SMTPHandler):
    def __init__(
        self,
        mailhost,
        fromaddr: str,
        toaddrs: Union[str, list[str]],
        subject: str,
        credentials: Optional[tuple[str, str]] = None,
        send_html: bool = False,
        backup_host: str = "",
        include_ssl_status: bool = False,
    ) -> None:
        super().__init__(mailhost, fromaddr, toaddrs, subject)
        self.send_html = send_html
        self.emailer = Emailer(
            (
                self.mailhost.split(":")[0]
                if isinstance(self.mailhost, str)
                else self.mailhost[0]
            ),
            self.mailport or 0,
            self.fromaddr,
            password=credentials[1] if credentials else None,
            use_keyring=False if credentials else True,
            unsecure_host=backup_host,
            include_ssl_status=include_ssl_status,
        )

    def emit(self, record: LogRecord) -> None:
        """
        Emit a record.

        Format the record and send it to the specified addressees.
        """
        try:
            result = self.emailer.send(
                self.toaddrs,
                self.getSubject(record),
                self.format(record),
                html=self.send_html,
            )
            if result != "success":
                print(result)
        except Exception:
            self.handleError(record)



from __future__ import annotations

import json
import sys
from pathlib import Path
from random import choice
from string import ascii_letters

src_path = str(Path(__file__).absolute().parent.parent)
sys.path.append(src_path)

from src.secure_email import Emailer

with open("config.json", "r") as f:
    config = json.load(f)


args = [
    config.get("host", ""),
    config.get("port", 0),
    config.get("from", ""),
]

kwargs = {
    "unsecure_host": config.get("bu_host", ""),
    "include_ssl_status": True,
}

pw = config.get("password")
kwargs["user"] = config.get("user")
if pw:
    kwargs["password"] = pw
else:
    kwargs["use_keyring"] = True

emailer = Emailer(
    # Either supply a password or set use_keyring to True to use password saved in keyring
    # See readme for instructions on storing password in keyring
    *args,
    **kwargs,
)

print(emailer.__dict__)

directory = Path(__file__).absolute().parent.joinpath("attachments")

txt = str(directory.joinpath("text_document.txt"))
img = str(directory.joinpath("image.png"))

menu = [
    "-" * 50,
    "Test email type (or <Enter> to exit):",
    "-" * 50,
    "",
    "1. Plain text email",
    "2. HTML email",
    "3. Email with attachment",
    "4. Email with multiple attachments",
    "5. Realy long email",
    "-" * 50,
    "Your response: ",
]


def randWord():
    limits = [i for i in range(3, 8)]
    return "".join([choice(ascii_letters) for _ in range(choice(limits))])


def randText(n):
    text = ""
    while len(text) < n:
        text = f"{text} {randWord()}".strip()
    return text[:n]


def printResult(result):
    success = result == "success"
    if not success:
        print(result)
    print(f"Email {'sent successfully' if success else 'failed'}")


while True:
    response = input("\n".join(menu))
    if response == "1":
        result = emailer.send(
            config.get("to", ""),
            "Plain text email",
            "Here is some plain text",
        )
        printResult(result)
    elif response == "2":
        result = emailer.send(
            config.get("to", ""),
            "HTML email",
            """
            <table role="presentation" border="0" align="center" cellpadding="12" width="600">
                <tr style="background-color: #008835; color: #fff; border: none;">
                    <h2 style="text-align: center; border: none;">Green header</h2>
                </tr>
            </table>
            <table
                role="presentation"
                border="1"
                cellpadding="6"
                align="center"
                width="600"
                border="0"
                style="border-collapse: collapse; background-color: #bddef1; color: #000;"
            >
                <tbody>
                    <tr><th>Row 1</th><td>Value 1</td></tr>
                    <tr><th>Row 2</th><td>Value 2</td></tr>
                    <tr><th>Row 3</th><td>Value 3</td></tr>
                </tbody>
            </table>
            """,
            html=True,
        )
        printResult(result)
    elif response == "3":
        result = emailer.send(
            config.get("to", ""),
            "Email with attachment",
            "Here is the gibberish you requested.",
            attachments=[txt],
        )
        printResult(result)
    elif response == "4":
        result = emailer.send(
            config.get("to", ""),
            "Email with multiple attachment",
            "Here are the random files you requested.",
            attachments=[txt, img],
        )
        printResult(result)
    elif response == "5":
        result = emailer.send(
            config.get("to", ""), "Really long email", randText(40_000)
        )
        printResult(result)
    if response == "":
        sys.exit()



import logging
import platform
import sys
import traceback
from functools import wraps
from pathlib import Path
from time import perf_counter
from typing import Any, Callable, Optional

from config import Config
from flask import Flask, jsonify, render_template, request
from flask_cors import CORS

src_path = str(Path(__file__).absolute().parent.parent.parent)
sys.path.append(src_path)

from src.secure_email import EmailHandler

app = Flask(__name__)
CORS(app)
app.config.from_object(Config)

from_: str = app.config.get("FROM", "")
to: str = app.config.get("TO", "")
host: str = app.config.get("HOST", "")
bu_host: str = app.config.get("BU_HOST", "")
user: str = f"lacsd.org\\{from_.split('@')[0]}"
port: int = app.config.get("PORT", 0)

logger = logging.getLogger("app")
logger.setLevel(logging.DEBUG)

default_formatter = logging.Formatter("%(levelname)s [%(asctime)s]: %(message)s")
email_formatter = logging.Formatter(
    "<p>%(levelname)s [%(asctime)s]:</p><p>%(message)s</p>"
)

fh = logging.FileHandler("logs.log")
fh.setLevel(logging.INFO)
fh.setFormatter(default_formatter)
mh = EmailHandler(
    (host, port),
    from_,
    to,
    "flask_mail_logger",
    send_html=True,
    backup_host=bu_host,
)
mh.setLevel(logging.ERROR)
mh.setFormatter(default_formatter)

logger.addHandler(fh)
logger.addHandler(mh)


def logRequest(func: Callable[..., Any]) -> Callable[..., Any]:
    @wraps(func)
    def wrapper(*args: Any, **kwargs: Any):
        result: Any
        try:
            start = perf_counter()
            result = func(*args, **kwargs)
            end = perf_counter()
            logger.info(f"{func.__name__} executed in {end - start:,.5f} seconds")
        except Exception as e:
            logger.error(
                f"{func.__name__} failed; {type(e).__name__}: {e} ({traceback.format_exc()})"
            )
            result = jsonify({type(e).__name__: str(e)})
        return result

    return wrapper


@app.route("/")
@logRequest
def home():
    return render_template("index.html")


@app.route("/api/set-number", methods=["POST"])
@logRequest
def setNumber():
    data: Optional[dict[str, Any]] = request.json
    if data:
        number: int = int(data.get("number", 0))
        # The line below ensures error if user enters 0 (or blank, since default is 0).
        # This is just to test the email handler, since the log level is set to ERROR
        _ = 2 / number
        return jsonify({"msg": f"Number {number} stored in database"})
    else:
        return jsonify({"msg": "No data received; nothing to update"})


if __name__ == "__main__":
    pc: str = platform.node()
    app.run(host=f"{pc}.lacsd.org", port=8888, debug=False)
